from enum import IntEnum
from typing import Generic, Optional, TypeVar

from pydantic import BaseModel, Field, NonNegativeInt


StatusT = TypeVar("StatusT")
ColorT = TypeVar("ColorT")


class Status(BaseModel, Generic[StatusT]):
    status: StatusT


class FadeTime(BaseModel):
    fade_time: float = Field(alias="fadeTime")


class DimmableWithFade(BaseModel):
    dim_value: float = Field(alias="dimValue")
    fade_time: float = Field(alias="fadeTime")


class ColorWithFadeTime(BaseModel, Generic[ColorT]):
    color: ColorT
    fade_time: float = Field(alias="fadeTime")


class ColorRGB(BaseModel):
    red: float = Field(alias="r")
    green: float = Field(alias="g")
    blue: float = Field(alias="b")
    dimmable: Optional[float] = None


class ColorWAF(BaseModel):
    white: float = Field(alias="w")
    amber: float = Field(alias="a")
    free_color: float = Field(alias="f")
    dimmable: Optional[float] = None


class ColorKelvinDimmable(BaseModel):
    dimmable: Optional[float] = None
    kelvin: NonNegativeInt


class ColorXY(BaseModel):
    x: float
    y: float
    dimmable: Optional[float] = None


class TimeSignature(BaseModel):
    timestamp: float
    counter: int


class FeaturesStatus(BaseModel):
    switchable: Optional[Status[bool]] = None
    dimmable: Optional[Status[float]] = None
    dimmable_with_fade: Optional[Status[DimmableWithFade]] = Field(
        None, alias="dimmableWithFade"
    )
    dim_up: Optional[bool] = Field(None, alias="dimUp")
    dim_down: Optional[bool] = Field(None, alias="dimDown")
    scene: Optional[bool] = None
    scene_with_fade: Optional[Status[FadeTime]] = Field(None, alias="sceneWithFade")
    goto_last_active: Optional[dict] = Field(None, alias="gotoLastActive")
    goto_last_active_with_fade: Optional[Status[FadeTime]] = Field(
        None, alias="gotoLastActiveWithFade"
    )
    dali_cmd16: Optional[bool] = Field(None, alias="daliCmd16")
    fade_time: Optional[Status[float]] = Field(None, alias="fadeTime")
    fade_rate: Optional[Status[float]] = Field(None, alias="fadeRate")
    save_to_scene: Optional[bool] = Field(None, alias="saveToScene")
    color_rgb: Optional[Status[ColorRGB]] = Field(None, alias="colorRGB")
    dimmable_rgb: Optional[Status[ColorRGB]] = Field(None, alias="dimmableRGB")
    color_rgb_with_fade: Optional[Status[ColorWithFadeTime[ColorRGB]]] = Field(
        None, alias="colorRGBWithFade"
    )
    color_waf: Optional[Status[ColorWAF]] = Field(None, alias="colorWAF")
    dimmable_waf: Optional[Status[ColorWAF]] = Field(None, alias="dimmableWAF")
    color_waf_with_fade: Optional[Status[ColorWithFadeTime[ColorWAF]]] = Field(
        None, alias="colorWAFWithFade"
    )
    color_kelvin: Optional[Status[NonNegativeInt]] = Field(None, alias="colorKelvin")
    dimmable_kelvin: Optional[Status[ColorKelvinDimmable]] = Field(
        None, alias="dimmableKelvin"
    )
    color_kelvin_with_fade: Optional[Status[ColorWithFadeTime[NonNegativeInt]]] = Field(
        None, alias="colorKelvinWithFade"
    )
    color_xy: Optional[Status[ColorXY]] = Field(None, alias="colorXY")
    dimmable_xy: Optional[Status[ColorXY]] = Field(None, alias="dimmableXY")
    color_xy_with_fade: Optional[Status[ColorWithFadeTime[ColorXY | dict]]] = Field(
        None, alias="colorXYWithFade"
    )


class DALIType(IntEnum):
    DT0_FLUORESCENT_LAMPS = 0
    DT1_EMERGENCY_LIGHTING = 1
    DT2_DISCHARGE_LAMPS = 2
    DT3_LOW_VOLTAGE_HALOGEN_LAMPS = 3
    DT4_SUPPLY_VOLTAGE_CONTROLLER = 4
    DT5_CONVERSION_FROM_DIGITAL_SIGNAL_INTO_DC_VOLTAGE = 5
    DT6_LED_MODULES = 6
    DT7_SWITCHING_FUNCTION = 7
    DT8_COLOUR_CONTROL = 8
    DT9_SEQUENCER = 9



from lunatone_rest_api_client import Auth
from lunatone_rest_api_client.models import ControlData, ZoneData, ZonesData

_PATH = "zones"


class Zone:
    """Class that represents a Zone object in the API."""

    base_path: str = _PATH[:-1]

    def __init__(self, auth: Auth, data: ZoneData) -> None:
        """Initialize a Zone object."""
        self._auth = auth
        self._data = data

    @property
    def path(self) -> str:
        return f"{self.base_path}/{self.id}"

    @property
    def data(self) -> ZoneData:
        """Return the raw zone data."""
        return self._data

    @property
    def id(self) -> int:
        """Return the ID of the zone."""
        return self.data.id

    @property
    def name(self) -> str:
        """Return the name of the zone."""
        return self.data.name

    async def async_update(self) -> None:
        response = await self._auth.get(self.path)
        self._data = ZoneData.model_validate(await response.json())

    async def async_control(self, data: ControlData) -> None:
        json_data = data.model_dump(by_alias=True, exclude_none=True)
        await self._auth.post(f"{self.path}/control", json=json_data)


class Zones:
    """Class that represents a Zones object in the API."""

    path: str = _PATH
    _data: ZonesData = None

    def __init__(self, auth: Auth) -> None:
        """Initialize a Zones object."""
        self._auth = auth

    @property
    def data(self) -> ZonesData:
        """Return the raw zones data."""
        return self._data

    @property
    def zones(self) -> list[Zone]:
        if self.data:
            return [Zone(self._auth, zone) for zone in self.data.zones]
        return []

    async def async_update(self) -> None:
        response = await self._auth.get(self.path)
        self._data = ZonesData.model_validate(await response.json())



import aiohttp
import pytest
from aioresponses.core import aioresponses

from lunatone_rest_api_client import Auth, Info
from lunatone_rest_api_client.models import InfoData

from .common import INFO_DATA_RAW


@pytest.fixture
def info_data() -> InfoData:
    return InfoData.model_validate(INFO_DATA_RAW)


@pytest.mark.asyncio
async def test_info_update(
    aioresponses: aioresponses, base_url: str, info_data: InfoData
) -> None:
    json_data = info_data.model_dump(by_alias=True)
    aioresponses.get(f"{base_url}/{Info.path}", payload=json_data)

    async with aiohttp.ClientSession() as session:
        info = Info(Auth(session, base_url))
        await info.async_update()
        assert info.data == info_data



from enum import StrEnum

from pydantic import BaseModel, Field


class DescriptorData(BaseModel):
    lines: int = 0
    bufferSize: int = 0
    tickResolution: int = 0
    maxYnFrameSize: int = 0
    implementedMacros: list[int] = []
    deviceListSpecifier: int = 0
    protocolVersionMajor: int = 1
    protocolVersionMinor: int = 0
    powerSupplyImplemented: bool = False


class DeviceInfoData(BaseModel):
    serial: int
    gtin: int
    pcb: str
    article_number: int = Field(alias="articleNumber")
    article_info: str = Field("", alias="articleInfo")
    production_year: int = Field(alias="productionYear")
    production_week: int = Field(alias="productionWeek")


class LineStatus(StrEnum):
    OK = "ok"
    LOW_POWER = "lowPower"
    NO_POWER = "noPower"


class DALIBusData(BaseModel):
    send_blocked_initialize: bool = Field(alias="sendBlockedInitialize")
    send_blocked_quiescent: bool = Field(alias="sendBlockedQuiescent")
    send_blocked_macro_running: bool = Field(alias="sendBlockedMacroRunning")
    send_buffer_full: bool = Field(alias="sendBufferFull")
    line_status: LineStatus = Field(alias="lineStatus")


class InfoData(BaseModel):
    name: str
    version: str
    tier: str = "basic"
    emergency_light: bool = Field(False, alias="emergencyLight")
    node_red: bool = Field(False, alias="nodeRed")
    errors: dict[str, str] = {}
    descriptor: DescriptorData = DescriptorData()
    device: DeviceInfoData
    lines: dict[str, DALIBusData] = {}



from typing import ClassVar, Optional

from lunatone_rest_api_client import Auth
from lunatone_rest_api_client.models import InfoData


class Info:
    """Class that represents a info object in the API."""

    path: ClassVar[str] = "info"

    def __init__(self, auth: Auth) -> None:
        """Initialize an info object."""
        self._auth = auth
        self._data = None

    @property
    def data(self) -> Optional[InfoData]:
        """Return the raw info data."""
        return self._data

    @property
    def name(self) -> Optional[str]:
        """Return the name of the API interface."""
        if self.data:
            return self.data.name
        return None

    @property
    def version(self) -> Optional[str]:
        """Return the software version of the API interface."""
        if self.data:
            return self.data.version
        return None

    @property
    def serial_number(self) -> Optional[int]:
        """Return the serial number of the API interface."""
        if self.data:
            return self.data.device.serial
        return None

    async def async_update(self) -> None:
        response = await self._auth.get(self.path)
        self._data = InfoData.model_validate(await response.json())



from typing import Optional

from pydantic import BaseModel, ConfigDict, Field, NonNegativeFloat, NonNegativeInt


class WithFadeTimeData(BaseModel):
    fade_time: Optional[NonNegativeFloat] = Field(None, alias="fadeTime")


class DimmableWithFadeData(WithFadeTimeData):
    dim_value: float = Field(alias="dimValue", ge=0.0, le=100.0)


class SceneWithFadeData(WithFadeTimeData):
    scene: int = Field(ge=0, lt=16)


class ColorRGBData(BaseModel):
    """Data model for the ``colorRGB`` feature."""

    red: Optional[float] = Field(
        None,
        description="Relative red value in the [0, 1] interval.",
        alias="r",
        ge=0.0,
        le=1.0,
    )
    green: Optional[float] = Field(
        None,
        description="Relative green value in the [0, 1] interval.",
        alias="g",
        ge=0.0,
        le=1.0,
    )
    blue: Optional[float] = Field(
        None,
        description="Relative blue value in the [0, 1] interval.",
        alias="b",
        ge=0.0,
        le=1.0,
    )

    model_config = ConfigDict(
        json_schema_extra={"examples": [{"r": 0.0, "g": 0.5, "b": 1.0}]}
    )


class DimmableRGBData(ColorRGBData):
    dimmable: float = Field(
        50.0,
        description="Percentage in the [0, 100] interval to dim a device to.",
        ge=0.0,
        le=100.0,
    )


class ColorRGBWithFadeData(WithFadeTimeData):
    color: ColorRGBData


class ColorWAFData(BaseModel):
    """Data model for the ``colorWAF`` feature."""

    white: Optional[float] = Field(
        None,
        description="Relative white value in the [0, 1] interval.",
        alias="w",
        ge=0.0,
        le=1.0,
    )
    amber: Optional[float] = Field(
        None,
        description="Relative amber value in the [0, 1] interval.",
        alias="a",
        ge=0.0,
        le=1.0,
    )
    free_color: Optional[float] = Field(
        None,
        description="Relative free color value in the [0, 1] interval.",
        alias="f",
        ge=0.0,
        le=1.0,
    )

    model_config = ConfigDict(
        json_schema_extra={"examples": [{"w": 0.0, "a": 0.5, "f": 1.0}]}
    )


class DimmableWAFData(ColorWAFData):
    dimmable: float = Field(
        50.0,
        description="Percentage in the [0, 100] interval to dim a device to.",
        ge=0.0,
        le=100.0,
    )


class ColorWAFWithFadeData(WithFadeTimeData):
    color: ColorWAFData


class DimmableKelvinData(BaseModel):
    dimmable: float = Field(
        50.0,
        description="Percentage in the [0, 100] interval to dim a device to.",
        ge=0.0,
        le=100.0,
    )
    kelvin: NonNegativeFloat = Field(4000.0, description="Color temperature in Kelvin.")


class ColorKelvinWithFadeData(WithFadeTimeData):
    color: NonNegativeFloat


class ColorXYData(BaseModel):
    """Data model for the ``colorXY`` feature."""

    x: Optional[float] = Field(
        None,
        description="X coordinate in the CIE color chromaticity space.",
    )
    y: Optional[float] = Field(
        None,
        description="Y coordinate in the CIE color chromaticity space.",
    )

    model_config = ConfigDict(
        json_schema_extra={"examples": [{"x": 0.432, "y": 0.150}]}
    )


class DimmableXYData(ColorXYData):
    dimmable: float = Field(
        50.0,
        description="Percentage in the [0, 100] interval to dim a device to.",
        ge=0.0,
        le=100.0,
    )


class ColorXYWithFadeData(WithFadeTimeData):
    color: ColorXYData


class ControlData(BaseModel):
    switchable: Optional[bool] = None
    dimmable: Optional[float] = Field(None, ge=0.0, le=100.0)
    dimmable_with_fade: Optional[DimmableWithFadeData] = Field(
        None, alias="dimmableWithFade"
    )
    dim_up: Optional[int] = Field(None, alias="dimUp", ge=1, le=1)
    dim_down: Optional[int] = Field(None, alias="dimDown", ge=1, le=1)
    goto_last_active: Optional[bool] = Field(
        None,
        description="Value must be ``true``.",
        alias="gotoLastActive",
    )
    goto_last_active_with_fade: Optional[WithFadeTimeData] = Field(
        None,
        description="Dim to the last level within a fade time in seconds.",
        alias="gotoLastActiveWithFade",
    )
    scene: Optional[int] = Field(
        None,
        description="Scene number of the scene to recall.",
        ge=0,
        lt=16,
    )
    scene_with_fade: Optional[SceneWithFadeData] = Field(
        None,
        description="Scene number of the scene to recall, within a fade time in seconds.",
        alias="sceneWithFade",
    )
    fade_time: Optional[NonNegativeFloat] = Field(
        None,
        description="Set the fade time in seconds.",
        alias="fadeTime",
    )
    fade_rate: Optional[NonNegativeFloat] = Field(
        None,
        description="Set the fade rate in steps per second.",
        alias="fadeRate",
    )
    save_to_scene: Optional[NonNegativeInt] = Field(
        None, alias="saveToScene", ge=0, lt=16
    )
    color_rgb: Optional[ColorRGBData] = Field(None, alias="colorRGB")
    dimmable_rgb: Optional[DimmableRGBData] = Field(None, alias="dimmableRGB")
    color_rgb_with_fade: Optional[ColorRGBWithFadeData] = Field(
        None, alias="colorRGBWithFade"
    )
    color_waf: Optional[ColorWAFData] = Field(None, alias="colorWAF")
    dimmable_waf: Optional[DimmableWAFData] = Field(None, alias="dimmableWAF")
    color_waf_with_fade: Optional[ColorWAFWithFadeData] = Field(
        None, alias="colorWAFWithFade"
    )
    color_kelvin: Optional[NonNegativeInt] = Field(
        None, alias="colorKelvin", gt=15, le=1000000
    )
    dimmable_kelvin: Optional[DimmableKelvinData] = Field(None, alias="dimmableKelvin")
    color_kelvin_with_fade: Optional[ColorKelvinWithFadeData] = Field(
        None, alias="colorKelvinWithFade"
    )
    color_xy: Optional[ColorXYData] = Field(None, alias="colorXY")
    dimmable_xy: Optional[DimmableXYData] = Field(None, alias="dimmableXY")
    color_xy_with_fade: Optional[ColorXYWithFadeData] = Field(
        None, alias="colorXYWithFade"
    )

    model_config = ConfigDict(
        json_schema_extra={
            "examples": [
                {
                    "gotoLastActive": True,
                    "gotoLastActiveWithFade": WithFadeTimeData(fadeTime=1.0),
                    "scene": 15,
                    "sceneWithFade": SceneWithFadeData(scene=15, fadeTime=1.0),
                    "fadeTime": 1.0,
                    "fadeRate": 15.8,
                }
            ]
        }
    )



from typing import Optional

from lunatone_rest_api_client import Auth
from lunatone_rest_api_client.models import (
    ControlData,
    DeviceData,
    DevicesData,
    DeviceUpdateData,
    TimeSignature,
)

_PATH = "devices"


class Device:
    """Class that represents a Device object in the API."""

    base_path: str = _PATH[:-1]

    def __init__(self, auth: Auth, data: DeviceData) -> None:
        """Initialize a device object."""
        self._auth = auth
        self._data = data

    @property
    def path(self) -> str:
        """Return the resource path."""
        return f"{self.base_path}/{self.id}"

    @property
    def data(self) -> DeviceData:
        """Return the raw device data."""
        return self._data

    @property
    def id(self) -> int:
        """Return the ID of the device."""
        return self.data.id

    @property
    def name(self) -> str:
        """Return the name of the device."""
        return self.data.name

    @property
    def time_signature(self) -> Optional[TimeSignature]:
        return self.data.time_signature

    @property
    def is_on(self) -> bool:
        """Return if the light is on."""
        return self.data.features.switchable.status

    @property
    def brightness(self) -> Optional[float]:
        """Return the brightness of the light."""
        return self.data.features.dimmable.status

    @property
    def color_temperature(self) -> Optional[int]:
        """Return the color temperature of the light in kelvin."""
        return self.data.features.color_kelvin.status

    @property
    def rgb_color(self) -> Optional[tuple[float, float, float]]:
        """Return the RGB color of the light as tuple."""
        return (
            self.data.features.color_rgb.status.red,
            self.data.features.color_rgb.status.green,
            self.data.features.color_rgb.status.blue,
        )

    @property
    def rgbw_color(self) -> Optional[tuple[float, float, float, float]]:
        """Return the RGBW color of the light as tuple."""
        return self.rgb_color + (self.data.features.color_waf.status.white)

    @property
    def xy_color(self) -> Optional[tuple[float, float]]:
        """Return the XY color of the light as tuple."""
        return (
            self.data.features.color_xy.status.x,
            self.data.features.color_xy.status.y,
        )

    async def async_update(self, data: DeviceUpdateData = None) -> None:
        """Update the device data."""
        if data is not None:
            json_data = data.model_dump(by_alias=True, exclude_none=True)
            response = await self._auth.put(self.path, json=json_data)
        else:
            response = await self._auth.get(self.path)
        self._data = DeviceData.model_validate(await response.json())

    async def async_control(self, data: ControlData) -> None:
        """Control the device."""
        json_data = data.model_dump(by_alias=True, exclude_none=True)
        await self._auth.post(f"{self.path}/control", json=json_data)


class Devices:
    """Class that represents a Devices object in the API."""

    path: str = _PATH
    _data: DevicesData = None

    def __init__(self, auth: Auth) -> None:
        """Initialize a Devices object."""
        self._auth = auth

    @property
    def data(self) -> Optional[DevicesData]:
        """Return the raw devices data."""
        return self._data

    @property
    def devices(self) -> list[Device]:
        if self.data:
            return [Device(self._auth, device) for device in self.data.devices]
        return []

    @property
    def time_signature(self) -> Optional[TimeSignature]:
        return self.data.time_signature if self.data else None

    async def async_update(self) -> None:
        response = await self._auth.get(self.path)
        self._data = DevicesData.model_validate(await response.json())



from typing import Optional

from pydantic import BaseModel, Field, PositiveInt, NonNegativeInt

from lunatone_rest_api_client.models import FeaturesStatus, TimeSignature


class ZoneTarget(BaseModel):
    type: str
    id: Optional[NonNegativeInt] = None


class ZoneData(BaseModel):
    id: PositiveInt
    name: str = ""
    targets: list[ZoneTarget] = []
    features: Optional[FeaturesStatus] = None
    time_signature: Optional[TimeSignature] = Field(None, alias="timeSignature")


class ZonesData(BaseModel):
    zones: list[ZoneData]
    time_signature: TimeSignature = Field(alias="timeSignature")



from typing import Optional

from pydantic import BaseModel, Field, NonNegativeInt, PositiveInt

from lunatone_rest_api_client.models import (
    DALIType,
    FeaturesStatus,
    TimeSignature,
)


class DeviceStatus(BaseModel):
    raw: NonNegativeInt = 0
    control_gear_failure: bool = Field(False, alias="controlGearFailure")
    lamp_failure: bool = Field(False, alias="lampFailure")
    lamp_on: bool = Field(False, alias="lampOn")
    limit_error: bool = Field(False, alias="limitError")
    fade_running: bool = Field(False, alias="fadeRunning")
    reset_state: bool = Field(False, alias="resetState")
    is_unaddressed: bool = Field(False, alias="isUnaddressed")
    power_cycle_see: bool = Field(False, alias="powerCycleSeen")


class DeviceData(BaseModel):
    id: PositiveInt
    name: str = ""
    type: str = "default"
    available: Optional[bool] = None
    status: Optional[DeviceStatus] = None
    features: Optional[FeaturesStatus] = None
    scenes: list = []
    groups: list[NonNegativeInt] = []
    address: Optional[NonNegativeInt] = None
    line: Optional[NonNegativeInt] = None
    dali_types: list[DALIType | NonNegativeInt] = Field([], alias="daliTypes")
    time_signature: Optional[TimeSignature] = Field(None, alias="timeSignature")


class DevicesData(BaseModel):
    devices: list[DeviceData]
    time_signature: Optional[TimeSignature] = Field(None, alias="timeSignature")


class DeviceUpdateData(BaseModel):
    name: str
    groups: Optional[list[NonNegativeInt]] = None
